from setuptools import setup, find_packages

setup(
    name="paddle-run",
    version="1.0.0",
    author="XiaosYu",
    author_email="lijingyu_ai@163.com",
    description="A distributed batch parameter running framework",
    long_description=open("README.md", encoding='utf-8').read(),
    long_description_content_type="text/markdown",
    url="https://github.com/xiaosyu/paddle-run",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
    install_requires=[
        "fastapi",
        "pickle",
        "requests",
        "websockets"
    ],
    entry_points={
        'console_scripts': [

        ],
    },
)